# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

import os

from ....base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)


class UKFTractographyInputSpec(CommandLineInputSpec):
    dwiFile = File(desc="Input DWI volume", exists=True, argstr="--dwiFile %s")
    seedsFile = File(
        desc="Seeds for diffusion. If not specified, full brain tractography will be performed, and the algorithm will start from every voxel in the brain mask where the Generalized Anisotropy is bigger than 0.18",
        exists=True,
        argstr="--seedsFile %s",
    )
    labels = InputMultiPath(
        traits.Int,
        desc="A vector of the ROI labels to be used",
        sep=",",
        argstr="--labels %s",
    )
    maskFile = File(
        desc="Mask for diffusion tractography", exists=True, argstr="--maskFile %s"
    )
    tracts = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Tracts generated, with first tensor output",
        argstr="--tracts %s",
    )
    writeAsciiTracts = traits.Bool(
        desc="Write tract file as a VTK binary data file", argstr="--writeAsciiTracts "
    )
    writeUncompressedTracts = traits.Bool(
        desc="Write tract file as a VTK uncompressed data file",
        argstr="--writeUncompressedTracts ",
    )
    seedsPerVoxel = traits.Int(
        desc=" Each seed generates a fiber, thus using more seeds generates more fibers. In general use 1 or 2 seeds, and for a more thorough result use 5 or 10 (depending on your machine this may take up to 2 days to run).,       ",
        argstr="--seedsPerVoxel %d",
    )
    numTensor = traits.Enum(
        "1", "2", desc="Number of tensors used", argstr="--numTensor %s"
    )
    freeWater = traits.Bool(
        desc="Adds a term for free water difusion to the model. (Note for experts: if checked, the 1T simple model is forced) ",
        argstr="--freeWater ",
    )
    recordFA = traits.Bool(
        desc="Whether to store FA. Attaches field 'FA', and 'FA2' for 2-tensor case to fiber. ",
        argstr="--recordFA ",
    )
    recordFreeWater = traits.Bool(
        desc="Whether to store the fraction of free water. Attaches field 'FreeWater' to fiber.",
        argstr="--recordFreeWater ",
    )
    recordTrace = traits.Bool(
        desc="Whether to store Trace. Attaches field 'Trace', and 'Trace2' for 2-tensor case to fiber.",
        argstr="--recordTrace ",
    )
    recordTensors = traits.Bool(
        desc="Recording the tensors enables Slicer to color the fiber bundles by FA, orientation, and so on. The fields will be called 'TensorN', where N is the tensor number. ",
        argstr="--recordTensors ",
    )
    recordNMSE = traits.Bool(
        desc="Whether to store NMSE. Attaches field 'NMSE' to fiber. ",
        argstr="--recordNMSE ",
    )
    recordState = traits.Bool(
        desc="Whether to attach the states to the fiber. Will generate field 'state'.",
        argstr="--recordState ",
    )
    recordCovariance = traits.Bool(
        desc="Whether to store the covariance. Will generate field 'covariance' in fiber.",
        argstr="--recordCovariance ",
    )
    recordLength = traits.Float(
        desc="Record length of tractography, in millimeters", argstr="--recordLength %f"
    )
    minFA = traits.Float(
        desc="Abort the tractography when the Fractional Anisotropy is less than this value",
        argstr="--minFA %f",
    )
    minGA = traits.Float(
        desc="Abort the tractography when the Generalized Anisotropy is less than this value",
        argstr="--minGA %f",
    )
    fullTensorModel = traits.Bool(
        desc="Whether to use the full tensor model. If unchecked, use the default simple tensor model",
        argstr="--fullTensorModel ",
    )
    numThreads = traits.Int(
        desc="Number of threads used during computation. Set to the number of cores on your workstation for optimal speed. If left undefined the number of cores detected will be used. ",
        argstr="--numThreads %d",
    )
    stepLength = traits.Float(
        desc="Step length of tractography, in millimeters", argstr="--stepLength %f"
    )
    maxHalfFiberLength = traits.Float(
        desc="The max length limit of the half fibers generated during tractography. Here the fiber is 'half' because the tractography goes in only one direction from one seed point at a time",
        argstr="--maxHalfFiberLength %f",
    )
    seedFALimit = traits.Float(
        desc="Seed points whose FA are below this value are excluded",
        argstr="--seedFALimit %f",
    )
    Qm = traits.Float(desc="Process noise for angles/direction", argstr="--Qm %f")
    Ql = traits.Float(desc="Process noise for eigenvalues", argstr="--Ql %f")
    Qw = traits.Float(
        desc="Process noise for free water weights, ignored if no free water estimation",
        argstr="--Qw %f",
    )
    Rs = traits.Float(desc="Measurement noise", argstr="--Rs %f")
    maxBranchingAngle = traits.Float(
        desc="Maximum branching angle, in degrees. When using multiple tensors, a new branch will be created when the tensors' major directions form an angle between (minBranchingAngle, maxBranchingAngle). Branching is supressed when this maxBranchingAngle is set to 0.0",
        argstr="--maxBranchingAngle %f",
    )
    minBranchingAngle = traits.Float(
        desc="Minimum branching angle, in degrees. When using multiple tensors, a new branch will be created when the tensors' major directions form an angle between (minBranchingAngle, maxBranchingAngle)",
        argstr="--minBranchingAngle %f",
    )
    tractsWithSecondTensor = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Tracts generated, with second tensor output (if there is one)",
        argstr="--tractsWithSecondTensor %s",
    )
    storeGlyphs = traits.Bool(
        desc="Store tensors' main directions as two-point lines in a separate file named glyphs_{tracts}. When using multiple tensors, only the major tensors' main directions are stored",
        argstr="--storeGlyphs ",
    )


class UKFTractographyOutputSpec(TraitedSpec):
    tracts = File(desc="Tracts generated, with first tensor output", exists=True)
    tractsWithSecondTensor = File(
        desc="Tracts generated, with second tensor output (if there is one)",
        exists=True,
    )


class UKFTractography(SEMLikeCommandLine):
    """title: UKF Tractography

category: Diffusion.Tractography

description: This module traces fibers in a DWI Volume using the multiple tensor unscented Kalman Filter methology. For more informations check the documentation.

version: 1.0

documentation-url: http://www.nitrc.org/plugins/mwiki/index.php/ukftractography:MainPage

contributor: Yogesh Rathi, Stefan Lienhard, Yinpeng Li, Martin Styner, Ipek Oguz, Yundi Shi, Christian Baumgartner, Kent Williams, Hans Johnson, Peter Savadjiev, Carl-Fredrik Westin.

acknowledgements: The development of this module was supported by NIH grants R01 MH097979 (PI Rathi), R01 MH092862 (PIs Westin and Verma), U01 NS083223 (PI Westin), R01 MH074794 (PI Westin) and P41 EB015902 (PI Kikinis).

"""

    input_spec = UKFTractographyInputSpec
    output_spec = UKFTractographyOutputSpec
    _cmd = " UKFTractography "
    _outputs_filenames = {
        "tracts": "tracts.vtp",
        "tractsWithSecondTensor": "tractsWithSecondTensor.vtp",
    }
    _redirect_x = False



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Test the resource management of MultiProc
"""
import sys
import os
import pytest
from nipype.pipeline import engine as pe
from nipype.interfaces import base as nib


class InputSpec(nib.TraitedSpec):
    input1 = nib.traits.Int(desc="a random int")
    input2 = nib.traits.Int(desc="a random int")


class OutputSpec(nib.TraitedSpec):
    output1 = nib.traits.List(nib.traits.Int, desc="outputs")


class MultiprocTestInterface(nib.BaseInterface):
    input_spec = InputSpec
    output_spec = OutputSpec

    def _run_interface(self, runtime):
        runtime.returncode = 0
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["output1"] = [1, self.inputs.input1]
        return outputs


@pytest.mark.skipif(
    sys.version_info >= (3, 8), reason="multiprocessing issues in Python 3.8"
)
def test_run_multiproc(tmpdir):
    tmpdir.chdir()

    pipe = pe.Workflow(name="pipe")
    mod1 = pe.Node(MultiprocTestInterface(), name="mod1")
    mod2 = pe.MapNode(MultiprocTestInterface(), iterfield=["input1"], name="mod2")
    pipe.connect([(mod1, mod2, [("output1", "input1")])])
    pipe.base_dir = os.getcwd()
    mod1.inputs.input1 = 1
    pipe.config["execution"]["poll_sleep_duration"] = 2
    execgraph = pipe.run(plugin="MultiProc")
    names = [node.fullname for node in execgraph.nodes()]
    node = list(execgraph.nodes())[names.index("pipe.mod1")]
    result = node.get_output("output1")
    assert result == [1, 1]


class InputSpecSingleNode(nib.TraitedSpec):
    input1 = nib.traits.Int(desc="a random int")
    input2 = nib.traits.Int(desc="a random int")


class OutputSpecSingleNode(nib.TraitedSpec):
    output1 = nib.traits.Int(desc="a random int")


class SingleNodeTestInterface(nib.BaseInterface):
    input_spec = InputSpecSingleNode
    output_spec = OutputSpecSingleNode

    def _run_interface(self, runtime):
        runtime.returncode = 0
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["output1"] = self.inputs.input1
        return outputs


def test_no_more_memory_than_specified(tmpdir):
    tmpdir.chdir()
    pipe = pe.Workflow(name="pipe")
    n1 = pe.Node(SingleNodeTestInterface(), name="n1", mem_gb=1)
    n2 = pe.Node(SingleNodeTestInterface(), name="n2", mem_gb=1)
    n3 = pe.Node(SingleNodeTestInterface(), name="n3", mem_gb=1)
    n4 = pe.Node(SingleNodeTestInterface(), name="n4", mem_gb=1)

    pipe.connect(n1, "output1", n2, "input1")
    pipe.connect(n1, "output1", n3, "input1")
    pipe.connect(n2, "output1", n4, "input1")
    pipe.connect(n3, "output1", n4, "input2")
    n1.inputs.input1 = 1

    max_memory = 0.5
    with pytest.raises(RuntimeError):
        pipe.run(
            plugin="MultiProc", plugin_args={"memory_gb": max_memory, "n_procs": 2}
        )


def test_no_more_threads_than_specified(tmpdir):
    tmpdir.chdir()

    pipe = pe.Workflow(name="pipe")
    n1 = pe.Node(SingleNodeTestInterface(), name="n1", n_procs=2)
    n2 = pe.Node(SingleNodeTestInterface(), name="n2", n_procs=2)
    n3 = pe.Node(SingleNodeTestInterface(), name="n3", n_procs=4)
    n4 = pe.Node(SingleNodeTestInterface(), name="n4", n_procs=2)

    pipe.connect(n1, "output1", n2, "input1")
    pipe.connect(n1, "output1", n3, "input1")
    pipe.connect(n2, "output1", n4, "input1")
    pipe.connect(n3, "output1", n4, "input2")
    n1.inputs.input1 = 4

    max_threads = 2
    with pytest.raises(RuntimeError):
        pipe.run(plugin="MultiProc", plugin_args={"n_procs": max_threads})


@pytest.mark.skipif(
    sys.version_info >= (3, 8), reason="multiprocessing issues in Python 3.8"
)
def test_hold_job_until_procs_available(tmpdir):
    tmpdir.chdir()

    pipe = pe.Workflow(name="pipe")
    n1 = pe.Node(SingleNodeTestInterface(), name="n1", n_procs=2)
    n2 = pe.Node(SingleNodeTestInterface(), name="n2", n_procs=2)
    n3 = pe.Node(SingleNodeTestInterface(), name="n3", n_procs=2)
    n4 = pe.Node(SingleNodeTestInterface(), name="n4", n_procs=2)

    pipe.connect(n1, "output1", n2, "input1")
    pipe.connect(n1, "output1", n3, "input1")
    pipe.connect(n2, "output1", n4, "input1")
    pipe.connect(n3, "output1", n4, "input2")
    n1.inputs.input1 = 4

    max_threads = 2
    pipe.run(plugin="MultiProc", plugin_args={"n_procs": max_threads})



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..registration import AffineRegistration


def test_AffineRegistration_inputs():
    input_map = dict(
        FixedImageFileName=dict(
            argstr="%s",
            extensions=None,
            position=-2,
        ),
        MovingImageFileName=dict(
            argstr="%s",
            extensions=None,
            position=-1,
        ),
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixedsmoothingfactor=dict(
            argstr="--fixedsmoothingfactor %d",
        ),
        histogrambins=dict(
            argstr="--histogrambins %d",
        ),
        initialtransform=dict(
            argstr="--initialtransform %s",
            extensions=None,
        ),
        iterations=dict(
            argstr="--iterations %d",
        ),
        movingsmoothingfactor=dict(
            argstr="--movingsmoothingfactor %d",
        ),
        outputtransform=dict(
            argstr="--outputtransform %s",
            hash_files=False,
        ),
        resampledmovingfilename=dict(
            argstr="--resampledmovingfilename %s",
            hash_files=False,
        ),
        spatialsamples=dict(
            argstr="--spatialsamples %d",
        ),
        translationscale=dict(
            argstr="--translationscale %f",
        ),
    )
    inputs = AffineRegistration.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_AffineRegistration_outputs():
    output_map = dict(
        outputtransform=dict(
            extensions=None,
        ),
        resampledmovingfilename=dict(
            extensions=None,
        ),
    )
    outputs = AffineRegistration.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
"""Provides interfaces to various commands provided by Camino-Trackvis."""

import os

from ...utils.filemanip import split_filename
from ..base import CommandLineInputSpec, CommandLine, traits, TraitedSpec, File


class Camino2TrackvisInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr="-i %s",
        mandatory=True,
        position=1,
        desc="The input .Bfloat (camino) file.",
    )

    out_file = File(
        argstr="-o %s",
        genfile=True,
        position=2,
        desc="The filename to which to write the .trk (trackvis) file.",
    )

    min_length = traits.Float(
        argstr="-l %d",
        position=3,
        units="mm",
        desc="The minimum length of tracts to output",
    )

    data_dims = traits.List(
        traits.Int,
        argstr="-d %s",
        sep=",",
        mandatory=True,
        position=4,
        minlen=3,
        maxlen=3,
        desc="Three comma-separated integers giving the number of voxels along each dimension of the source scans.",
    )

    voxel_dims = traits.List(
        traits.Float,
        argstr="-x %s",
        sep=",",
        mandatory=True,
        position=5,
        minlen=3,
        maxlen=3,
        desc="Three comma-separated numbers giving the size of each voxel in mm.",
    )

    # Change to enum with all combinations? i.e. LAS, LPI, RAS, etc..
    voxel_order = File(
        argstr="--voxel-order %s",
        mandatory=True,
        position=6,
        desc="Set the order in which various directions were stored.\
        Specify with three letters consisting of one each  \
        from the pairs LR, AP, and SI. These stand for Left-Right, \
        Anterior-Posterior, and Superior-Inferior.  \
        Whichever is specified in each position will  \
        be the direction of increasing order.  \
        Read coordinate system from a NIfTI file.",
    )

    nifti_file = File(
        argstr="--nifti %s",
        exists=True,
        position=7,
        desc="Read coordinate system from a NIfTI file.",
    )


class Camino2TrackvisOutputSpec(TraitedSpec):
    trackvis = File(
        exists=True, desc="The filename to which to write the .trk (trackvis) file."
    )


class Camino2Trackvis(CommandLine):
    """ Wraps camino_to_trackvis from Camino-Trackvis

    Convert files from camino .Bfloat format to trackvis .trk format.

    Example
    -------

    >>> import nipype.interfaces.camino2trackvis as cam2trk
    >>> c2t = cam2trk.Camino2Trackvis()
    >>> c2t.inputs.in_file = 'data.Bfloat'
    >>> c2t.inputs.out_file = 'streamlines.trk'
    >>> c2t.inputs.min_length = 30
    >>> c2t.inputs.data_dims = [128, 104, 64]
    >>> c2t.inputs.voxel_dims = [2.0, 2.0, 2.0]
    >>> c2t.inputs.voxel_order = 'LAS'
    >>> c2t.run()                  # doctest: +SKIP
    """

    _cmd = "camino_to_trackvis"
    input_spec = Camino2TrackvisInputSpec
    output_spec = Camino2TrackvisOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs["trackvis"] = os.path.abspath(self._gen_outfilename())
        return outputs

    def _gen_filename(self, name):
        if name == "out_file":
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_file)
        return name + ".trk"


class Trackvis2CaminoInputSpec(CommandLineInputSpec):
    """ Wraps trackvis_to_camino from Camino-Trackvis

    Convert files from camino .Bfloat format to trackvis .trk format.

    Example
    -------

    >>> import nipype.interfaces.camino2trackvis as cam2trk
    >>> t2c = cam2trk.Trackvis2Camino()
    >>> t2c.inputs.in_file = 'streamlines.trk'
    >>> t2c.inputs.out_file = 'streamlines.Bfloat'
    >>> t2c.run()                  # doctest: +SKIP
    """

    in_file = File(
        exists=True,
        argstr="-i %s",
        mandatory=True,
        position=1,
        desc="The input .trk (trackvis) file.",
    )

    out_file = File(
        argstr="-o %s",
        genfile=True,
        position=2,
        desc="The filename to which to write the .Bfloat (camino).",
    )

    append_file = File(
        exists=True,
        argstr="-a %s",
        position=2,
        desc="A file to which the append the .Bfloat data. ",
    )


class Trackvis2CaminoOutputSpec(TraitedSpec):
    camino = File(
        exists=True, desc="The filename to which to write the .Bfloat (camino)."
    )


class Trackvis2Camino(CommandLine):
    _cmd = "trackvis_to_camino"
    input_spec = Trackvis2CaminoInputSpec
    output_spec = Trackvis2CaminoOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs["camino"] = os.path.abspath(self._gen_outfilename())
        return outputs

    def _gen_filename(self, name):
        if name == "out_file":
            return self._gen_outfilename()
        else:
            return None

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_file)
        return name + ".Bfloat"



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..qt1 import FitQt1


def test_FitQt1_inputs():
    input_map = dict(
        acceptance=dict(
            argstr="-acceptance %f",
        ),
        args=dict(
            argstr="%s",
        ),
        b1map=dict(
            argstr="-b1map %s",
            extensions=None,
        ),
        comp_file=dict(
            argstr="-comp %s",
            extensions=None,
            name_source=["source_file"],
            name_template="%s_comp.nii.gz",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        error_file=dict(
            argstr="-error %s",
            extensions=None,
            name_source=["source_file"],
            name_template="%s_error.nii.gz",
        ),
        flips=dict(
            argstr="-flips %s",
            sep=" ",
        ),
        flips_list=dict(
            argstr="-fliplist %s",
            extensions=None,
        ),
        gn_flag=dict(
            argstr="-gn",
            position=8,
        ),
        ir_flag=dict(
            argstr="-IR",
            position=13,
        ),
        lm_val=dict(
            argstr="-lm %f %f",
            position=7,
        ),
        m0map_file=dict(
            argstr="-m0map %s",
            extensions=None,
            name_source=["source_file"],
            name_template="%s_m0map.nii.gz",
        ),
        mask=dict(
            argstr="-mask %s",
            extensions=None,
            position=2,
        ),
        maxit=dict(
            argstr="-maxit %d",
            position=11,
        ),
        mcmap_file=dict(
            argstr="-mcmap %s",
            extensions=None,
            name_source=["source_file"],
            name_template="%s_mcmap.nii.gz",
        ),
        mcmaxit=dict(
            argstr="-mcmaxit %d",
        ),
        mcout=dict(
            argstr="-mcout %s",
            extensions=None,
        ),
        mcsamples=dict(
            argstr="-mcsamples %d",
        ),
        nb_comp=dict(
            argstr="-nc %d",
            position=6,
        ),
        prior=dict(
            argstr="-prior %s",
            extensions=None,
            position=3,
        ),
        res_file=dict(
            argstr="-res %s",
            extensions=None,
            name_source=["source_file"],
            name_template="%s_res.nii.gz",
        ),
        slice_no=dict(
            argstr="-slice %d",
            position=9,
        ),
        source_file=dict(
            argstr="-source %s",
            extensions=None,
            mandatory=True,
            position=1,
        ),
        spgr=dict(
            argstr="-SPGR",
        ),
        sr_flag=dict(
            argstr="-SR",
            position=12,
        ),
        syn_file=dict(
            argstr="-syn %s",
            extensions=None,
            name_source=["source_file"],
            name_template="%s_syn.nii.gz",
        ),
        t1_list=dict(
            argstr="-T1list %s",
            extensions=None,
        ),
        t1map_file=dict(
            argstr="-t1map %s",
            extensions=None,
            name_source=["source_file"],
            name_template="%s_t1map.nii.gz",
        ),
        t1max=dict(
            argstr="-T1max %f",
        ),
        t1min=dict(
            argstr="-T1min %f",
        ),
        te_value=dict(
            argstr="-TE %f",
            position=4,
        ),
        tis=dict(
            argstr="-TIs %s",
            position=14,
            sep=" ",
        ),
        tis_list=dict(
            argstr="-TIlist %s",
            extensions=None,
        ),
        tr_value=dict(
            argstr="-TR %f",
            position=5,
        ),
        voxel=dict(
            argstr="-voxel %d %d %d",
            position=10,
        ),
    )
    inputs = FitQt1.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_FitQt1_outputs():
    output_map = dict(
        comp_file=dict(
            extensions=None,
        ),
        error_file=dict(
            extensions=None,
        ),
        m0map_file=dict(
            extensions=None,
        ),
        mcmap_file=dict(
            extensions=None,
        ),
        res_file=dict(
            extensions=None,
        ),
        syn_file=dict(
            extensions=None,
        ),
        t1map_file=dict(
            extensions=None,
        ),
    )
    outputs = FitQt1.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""This module provides interfaces for workbench CIFTI commands"""
from ..base import TraitedSpec, File, traits, CommandLineInputSpec
from .base import WBCommand
from ... import logging

iflogger = logging.getLogger("nipype.interface")


class CiftiSmoothInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        mandatory=True,
        argstr="%s",
        position=0,
        desc="The input CIFTI file",
    )
    sigma_surf = traits.Float(
        mandatory=True,
        argstr="%s",
        position=1,
        desc="the sigma for the gaussian surface smoothing kernel, in mm",
    )
    sigma_vol = traits.Float(
        mandatory=True,
        argstr="%s",
        position=2,
        desc="the sigma for the gaussian volume smoothing kernel, in mm",
    )
    direction = traits.Enum(
        "ROW",
        "COLUMN",
        mandatory=True,
        argstr="%s",
        position=3,
        desc="which dimension to smooth along, ROW or COLUMN",
    )
    out_file = File(
        name_source=["in_file"],
        name_template="smoothed_%s.nii",
        keep_extension=True,
        argstr="%s",
        position=4,
        desc="The output CIFTI",
    )
    left_surf = File(
        exists=True,
        mandatory=True,
        position=5,
        argstr="-left-surface %s",
        desc="Specify the left surface to use",
    )
    left_corrected_areas = File(
        exists=True,
        position=6,
        argstr="-left-corrected-areas %s",
        desc="vertex areas (as a metric) to use instead of computing them from "
        "the left surface.",
    )
    right_surf = File(
        exists=True,
        mandatory=True,
        position=7,
        argstr="-right-surface %s",
        desc="Specify the right surface to use",
    )
    right_corrected_areas = File(
        exists=True,
        position=8,
        argstr="-right-corrected-areas %s",
        desc="vertex areas (as a metric) to use instead of computing them from "
        "the right surface",
    )
    cerebellum_surf = File(
        exists=True,
        position=9,
        argstr="-cerebellum-surface %s",
        desc="specify the cerebellum surface to use",
    )
    cerebellum_corrected_areas = File(
        exists=True,
        position=10,
        requires=["cerebellum_surf"],
        argstr="cerebellum-corrected-areas %s",
        desc="vertex areas (as a metric) to use instead of computing them from "
        "the cerebellum surface",
    )
    cifti_roi = File(
        exists=True,
        position=11,
        argstr="-cifti-roi %s",
        desc="CIFTI file for ROI smoothing",
    )
    fix_zeros_vol = traits.Bool(
        position=12,
        argstr="-fix-zeros-volume",
        desc="treat values of zero in the volume as missing data",
    )
    fix_zeros_surf = traits.Bool(
        position=13,
        argstr="-fix-zeros-surface",
        desc="treat values of zero on the surface as missing data",
    )
    merged_volume = traits.Bool(
        position=14,
        argstr="-merged-volume",
        desc="smooth across subcortical structure boundaries",
    )


class CiftiSmoothOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc="output CIFTI file")


class CiftiSmooth(WBCommand):
    """
    Smooth a CIFTI file

    The input cifti file must have a brain models mapping on the chosen
    dimension, columns for .dtseries, and either for .dconn.  By default,
    data in different structures is smoothed independently (i.e., "parcel
    constrained" smoothing), so volume structures that touch do not smooth
    across this boundary.  Specify ``merged_volume`` to ignore these
    boundaries. Surface smoothing uses the ``GEO_GAUSS_AREA`` smoothing method.

    The ``*_corrected_areas`` options are intended for when it is unavoidable
    to smooth on group average surfaces, it is only an approximate correction
    for the reduction of structure in a group average surface.  It is better
    to smooth the data on individuals before averaging, when feasible.

    The ``fix_zeros_*`` options will treat values of zero as lack of data, and
    not use that value when generating the smoothed values, but will fill
    zeros with extrapolated values.  The ROI should have a brain models
    mapping along columns, exactly matching the mapping of the chosen
    direction in the input file.  Data outside the ROI is ignored.

    >>> from nipype.interfaces.workbench import CiftiSmooth
    >>> smooth = CiftiSmooth()
    >>> smooth.inputs.in_file = 'sub-01_task-rest.dtseries.nii'
    >>> smooth.inputs.sigma_surf = 4
    >>> smooth.inputs.sigma_vol = 4
    >>> smooth.inputs.direction = 'COLUMN'
    >>> smooth.inputs.right_surf = 'sub-01.R.midthickness.32k_fs_LR.surf.gii'
    >>> smooth.inputs.left_surf = 'sub-01.L.midthickness.32k_fs_LR.surf.gii'
    >>> smooth.cmdline
    'wb_command -cifti-smoothing sub-01_task-rest.dtseries.nii 4.0 4.0 COLUMN \
    smoothed_sub-01_task-rest.dtseries.nii \
    -left-surface sub-01.L.midthickness.32k_fs_LR.surf.gii \
    -right-surface sub-01.R.midthickness.32k_fs_LR.surf.gii'
    """

    input_spec = CiftiSmoothInputSpec
    output_spec = CiftiSmoothOutputSpec
    _cmd = "wb_command -cifti-smoothing"



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import numpy as np
import nibabel as nb
import pytest
from ..imagemanip import copy_header


@pytest.mark.parametrize("keep_dtype", (True, False))
def test_copy_header(tmp_path, keep_dtype):
    """Cover copy_header."""
    fname1 = tmp_path / "reference.nii.gz"
    fname2 = tmp_path / "target.nii.gz"

    nii = nb.Nifti1Image(np.zeros((10, 10, 10), dtype="uint8"), None, None)
    nii.set_qform(np.diag((1.0, 2.0, 3.0, 1.0)), code=2)
    nii.set_sform(np.diag((1.0, 2.0, 3.0, 1.0)), code=1)
    nii.to_filename(str(fname1))

    nii.set_data_dtype("float32")
    nii.set_qform(np.eye(4), code=1)
    nii.to_filename(str(fname2))

    copied = nb.load(copy_header(fname1, fname2, keep_dtype=keep_dtype))
    ref = nb.load(str(fname1))
    assert np.all(copied.get_qform(coded=False) == ref.get_qform(coded=False))
    assert np.all(copied.get_sform(coded=False) == ref.get_sform(coded=False))
    assert copied.get_qform(coded=True)[1] == ref.get_qform(coded=True)[1]
    assert copied.get_sform(coded=True)[1] == ref.get_sform(coded=True)[1]
    assert (copied.header.get_data_dtype() == ref.header.get_data_dtype()) != keep_dtype



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Reformat interface docstrings."""
import re
from sphinxcontrib.napoleon._upstream import _
from sphinxcontrib.napoleon.docstring import NumpyDocstring


class NipypeDocstring(NumpyDocstring):
    """Patch the NumpyDocstring from napoleon to get special section headers."""

    def _parse_parameters_section(self, section):
        # type: (unicode) -> List[unicode]
        labels = {
            "args": _("Parameters"),
            "arguments": _("Parameters"),
            "parameters": _("Parameters"),
        }  # type: Dict[unicode, unicode]
        label = labels.get(section.lower(), section)

        fields = self._consume_fields()
        if self._config.napoleon_use_param:
            return self._format_docutils_params(fields)

        return self._format_fields(label, fields)


class InterfaceDocstring(NipypeDocstring):
    """
    Convert docstrings of Nipype Interfaces to reStructuredText.

    Parameters
    ----------
    docstring : :obj:`str` or :obj:`list` of :obj:`str`
        The docstring to parse, given either as a string or split into
        individual lines.
    config: :obj:`sphinxcontrib.napoleon.Config` or :obj:`sphinx.config.Config`
        The configuration settings to use. If not given, defaults to the
        config object on `app`; or if `app` is not given defaults to the
        a new :class:`nipype.sphinxext.apidoc.Config` object.

    Other Parameters
    ----------------
    app : :class:`sphinx.application.Sphinx`, optional
        Application object representing the Sphinx process.
    what : :obj:`str`, optional
        A string specifying the type of the object to which the docstring
        belongs. Valid values: "module", "class", "exception", "function",
        "method", "attribute".
    name : :obj:`str`, optional
        The fully qualified name of the object.
    obj : module, class, exception, function, method, or attribute
        The object to which the docstring belongs.
    options : :class:`sphinx.ext.autodoc.Options`, optional
        The options given to the directive: an object with attributes
        inherited_members, undoc_members, show_inheritance and noindex that
        are True if the flag option of same name was given to the auto
        directive.

    """

    _name_rgx = re.compile(
        r"^\s*(:(?P<role>\w+):`(?P<name>[a-zA-Z0-9_.-]+)`|"
        r" (?P<name2>[a-zA-Z0-9_.-]+))\s*",
        re.X,
    )

    def __init__(
        self, docstring, config=None, app=None, what="", name="", obj=None, options=None
    ):
        # type: (Union[unicode, List[unicode]], SphinxConfig, Sphinx, unicode, unicode, Any, Any) -> None  # NOQA
        super().__init__(docstring, config, app, what, name, obj, options)

        cmd = getattr(obj, "_cmd", "")
        if cmd and cmd.strip():
            self._parsed_lines = [
                "Wrapped executable: ``%s``." % cmd.strip(),
                "",
            ] + self._parsed_lines

        if obj is not None:
            self._parsed_lines += _parse_interface(obj)


def _parse_interface(obj):
    """Print description for input parameters."""
    parsed = []
    if obj.input_spec:
        inputs = obj.input_spec()
        mandatory_items = sorted(inputs.traits(mandatory=True).items())
        if mandatory_items:
            parsed += ["", "Mandatory Inputs"]
            parsed += ["-" * len(parsed[-1])]
            for name, spec in mandatory_items:
                parsed += _parse_spec(inputs, name, spec)

        mandatory_keys = {item[0] for item in mandatory_items}
        optional_items = sorted(
            [
                (name, val)
                for name, val in inputs.traits(transient=None).items()
                if name not in mandatory_keys
            ]
        )
        if optional_items:
            parsed += ["", "Optional Inputs"]
            parsed += ["-" * len(parsed[-1])]
            for name, spec in optional_items:
                parsed += _parse_spec(inputs, name, spec)

    if obj.output_spec:
        outputs = sorted(obj.output_spec().traits(transient=None).items())
        if outputs:
            parsed += ["", "Outputs"]
            parsed += ["-" * len(parsed[-1])]
            for name, spec in outputs:
                parsed += _parse_spec(inputs, name, spec)

    return parsed


def _indent(lines, n=4):
    # type: (List[unicode], int) -> List[unicode]
    return [(" " * n) + line for line in lines]


def _parse_spec(inputs, name, spec):
    """Parse a HasTraits object into a Numpy-style docstring."""
    desc_lines = []
    if spec.desc:
        desc = "".join([spec.desc[0].capitalize(), spec.desc[1:]])
        if not desc.endswith(".") and not desc.endswith("\n"):
            desc = "%s." % desc
        desc_lines += desc.splitlines()

    argstr = spec.argstr
    if argstr and argstr.strip():
        pos = spec.position
        if pos is None:
            desc_lines += [
                """Maps to a command-line argument: :code:`{arg}`.""".format(
                    arg=argstr.strip()
                )
            ]
        else:
            desc_lines += [
                """Maps to a command-line argument: :code:`{arg}` (position: {pos}).""".format(
                    arg=argstr.strip(), pos=pos
                )
            ]

    xor = spec.xor
    if xor:
        desc_lines += [
            "Mutually **exclusive** with inputs: %s."
            % ", ".join(["``%s``" % x for x in xor])
        ]

    requires = spec.requires
    if requires:
        desc_lines += [
            "**Requires** inputs: %s." % ", ".join(["``%s``" % x for x in requires])
        ]

    if spec.usedefault:
        default = spec.default_value()[1]
        if isinstance(default, (bytes, str)) and not default:
            default = '""'

        desc_lines += ["(Nipype **default** value: ``%s``)" % str(default)]

    out_rst = [
        "{name} : {type}".format(name=name, type=spec.full_info(inputs, name, None))
    ]
    out_rst += _indent(desc_lines, 4)

    return out_rst



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import os

import nipype.interfaces.fsl as fsl
from nipype.interfaces.base import InterfaceResult
from nipype.interfaces.fsl import check_fsl, no_fsl

import pytest


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_fslversion():
    ver = fsl.Info.version()
    assert ver.split(".", 1)[0].isdigit()


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_fsloutputtype():
    types = list(fsl.Info.ftypes.keys())
    orig_out_type = fsl.Info.output_type()
    assert orig_out_type in types


def test_outputtype_to_ext():
    for ftype, ext in fsl.Info.ftypes.items():
        res = fsl.Info.output_type_to_ext(ftype)
        assert res == ext

    with pytest.raises(KeyError):
        fsl.Info.output_type_to_ext("JUNK")


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_FSLCommand():
    # Most methods in FSLCommand are tested in the subclasses.  Only
    # testing the one item that is not.
    cmd = fsl.FSLCommand(command="ls")
    res = cmd.run()
    assert type(res) == InterfaceResult


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_FSLCommand2():
    # Check default output type and environ
    cmd = fsl.FSLCommand(command="junk")
    assert cmd._output_type == fsl.Info.output_type()
    assert cmd.inputs.environ["FSLOUTPUTTYPE"] == cmd._output_type
    assert cmd._output_type in fsl.Info.ftypes

    cmd = fsl.FSLCommand
    cmdinst = fsl.FSLCommand(command="junk")
    for out_type in fsl.Info.ftypes:
        cmd.set_default_output_type(out_type)
        assert cmd._output_type == out_type
        if out_type != fsl.Info.output_type():
            #  Setting class outputtype should not effect existing instances
            assert cmdinst.inputs.output_type != out_type


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
@pytest.mark.parametrize(
    "args, desired_name",
    [
        ({}, {"file": "foo.nii.gz"}),  # just the filename
        # filename with suffix
        ({"suffix": "_brain"}, {"file": "foo_brain.nii.gz"}),
        (
            {"suffix": "_brain", "cwd": "/data"},
            # filename with suffix and working directory
            {"dir": "/data", "file": "foo_brain.nii.gz"},
        ),
        # filename with suffix and no file extension change
        ({"suffix": "_brain.mat", "change_ext": False}, {"file": "foo_brain.mat"}),
    ],
)
def test_gen_fname(args, desired_name):
    # Test _gen_fname method of FSLCommand
    cmd = fsl.FSLCommand(command="junk", output_type="NIFTI_GZ")
    pth = os.getcwd()
    fname = cmd._gen_fname("foo.nii.gz", **args)
    if "dir" in desired_name.keys():
        desired = os.path.join(desired_name["dir"], desired_name["file"])
    else:
        desired = os.path.join(pth, desired_name["file"])
    assert fname == desired



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..epi import TOPUP


def test_TOPUP_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        config=dict(
            argstr="--config=%s",
            usedefault=True,
        ),
        encoding_direction=dict(
            argstr="--datain=%s",
            mandatory=True,
            requires=["readout_times"],
            xor=["encoding_file"],
        ),
        encoding_file=dict(
            argstr="--datain=%s",
            extensions=None,
            mandatory=True,
            xor=["encoding_direction"],
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        estmov=dict(
            argstr="--estmov=%d",
        ),
        fwhm=dict(
            argstr="--fwhm=%f",
        ),
        in_file=dict(
            argstr="--imain=%s",
            extensions=None,
            mandatory=True,
        ),
        interp=dict(
            argstr="--interp=%s",
        ),
        max_iter=dict(
            argstr="--miter=%d",
        ),
        minmet=dict(
            argstr="--minmet=%d",
        ),
        numprec=dict(
            argstr="--numprec=%s",
        ),
        out_base=dict(
            argstr="--out=%s",
            extensions=None,
            hash_files=False,
            name_source=["in_file"],
            name_template="%s_base",
        ),
        out_corrected=dict(
            argstr="--iout=%s",
            extensions=None,
            hash_files=False,
            name_source=["in_file"],
            name_template="%s_corrected",
        ),
        out_field=dict(
            argstr="--fout=%s",
            extensions=None,
            hash_files=False,
            name_source=["in_file"],
            name_template="%s_field",
        ),
        out_jac_prefix=dict(
            argstr="--jacout=%s",
            hash_files=False,
            usedefault=True,
        ),
        out_logfile=dict(
            argstr="--logout=%s",
            extensions=None,
            hash_files=False,
            keep_extension=True,
            name_source=["in_file"],
            name_template="%s_topup.log",
        ),
        out_mat_prefix=dict(
            argstr="--rbmout=%s",
            hash_files=False,
            usedefault=True,
        ),
        out_warp_prefix=dict(
            argstr="--dfout=%s",
            hash_files=False,
            usedefault=True,
        ),
        output_type=dict(),
        readout_times=dict(
            mandatory=True,
            requires=["encoding_direction"],
            xor=["encoding_file"],
        ),
        reg_lambda=dict(
            argstr="--lambda=%0.f",
        ),
        regmod=dict(
            argstr="--regmod=%s",
        ),
        regrid=dict(
            argstr="--regrid=%d",
        ),
        scale=dict(
            argstr="--scale=%d",
        ),
        splineorder=dict(
            argstr="--splineorder=%d",
        ),
        ssqlambda=dict(
            argstr="--ssqlambda=%d",
        ),
        subsamp=dict(
            argstr="--subsamp=%d",
        ),
        warp_res=dict(
            argstr="--warpres=%f",
        ),
    )
    inputs = TOPUP.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_TOPUP_outputs():
    output_map = dict(
        out_corrected=dict(
            extensions=None,
        ),
        out_enc_file=dict(
            extensions=None,
        ),
        out_field=dict(
            extensions=None,
        ),
        out_fieldcoef=dict(
            extensions=None,
        ),
        out_jacs=dict(),
        out_logfile=dict(
            extensions=None,
        ),
        out_mats=dict(),
        out_movpar=dict(
            extensions=None,
        ),
        out_warps=dict(),
    )
    outputs = TOPUP.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import LTAConvert


def test_LTAConvert_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_fsl=dict(
            argstr="--infsl %s",
            extensions=None,
            mandatory=True,
            xor=("in_lta", "in_fsl", "in_mni", "in_reg", "in_niftyreg", "in_itk"),
        ),
        in_itk=dict(
            argstr="--initk %s",
            extensions=None,
            mandatory=True,
            xor=("in_lta", "in_fsl", "in_mni", "in_reg", "in_niftyreg", "in_itk"),
        ),
        in_lta=dict(
            argstr="--inlta %s",
            mandatory=True,
            xor=("in_lta", "in_fsl", "in_mni", "in_reg", "in_niftyreg", "in_itk"),
        ),
        in_mni=dict(
            argstr="--inmni %s",
            extensions=None,
            mandatory=True,
            xor=("in_lta", "in_fsl", "in_mni", "in_reg", "in_niftyreg", "in_itk"),
        ),
        in_niftyreg=dict(
            argstr="--inniftyreg %s",
            extensions=None,
            mandatory=True,
            xor=("in_lta", "in_fsl", "in_mni", "in_reg", "in_niftyreg", "in_itk"),
        ),
        in_reg=dict(
            argstr="--inreg %s",
            extensions=None,
            mandatory=True,
            xor=("in_lta", "in_fsl", "in_mni", "in_reg", "in_niftyreg", "in_itk"),
        ),
        invert=dict(
            argstr="--invert",
        ),
        ltavox2vox=dict(
            argstr="--ltavox2vox",
            requires=["out_lta"],
        ),
        out_fsl=dict(
            argstr="--outfsl %s",
        ),
        out_itk=dict(
            argstr="--outitk %s",
        ),
        out_lta=dict(
            argstr="--outlta %s",
        ),
        out_mni=dict(
            argstr="--outmni %s",
        ),
        out_reg=dict(
            argstr="--outreg %s",
        ),
        source_file=dict(
            argstr="--src %s",
            extensions=None,
        ),
        target_conform=dict(
            argstr="--trgconform",
        ),
        target_file=dict(
            argstr="--trg %s",
            extensions=None,
        ),
    )
    inputs = LTAConvert.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_LTAConvert_outputs():
    output_map = dict(
        out_fsl=dict(
            extensions=None,
        ),
        out_itk=dict(
            extensions=None,
        ),
        out_lta=dict(
            extensions=None,
        ),
        out_mni=dict(
            extensions=None,
        ),
        out_reg=dict(
            extensions=None,
        ),
    )
    outputs = LTAConvert.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)
import os


class OtsuThresholdImageFilterInputSpec(CommandLineInputSpec):
    insideValue = traits.Int(
        desc="The value assigned to pixels that are inside the computed threshold",
        argstr="--insideValue %d",
    )
    outsideValue = traits.Int(
        desc="The value assigned to pixels that are outside the computed threshold",
        argstr="--outsideValue %d",
    )
    numberOfBins = traits.Int(
        desc="This is an advanced parameter. The number of bins in the histogram used to model the probability mass function of the two intensity distributions. Small numbers of bins may result in a more conservative threshold. The default should suffice for most applications. Experimentation is the only way to see the effect of varying this parameter.",
        argstr="--numberOfBins %d",
    )
    inputVolume = File(
        position=-2, desc="Input volume to be filtered", exists=True, argstr="%s"
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Output filtered",
        argstr="%s",
    )


class OtsuThresholdImageFilterOutputSpec(TraitedSpec):
    outputVolume = File(position=-1, desc="Output filtered", exists=True)


class OtsuThresholdImageFilter(SEMLikeCommandLine):
    """title: Otsu Threshold Image Filter

category: Legacy.Filtering

description: This filter creates a binary thresholded image that separates an image into foreground and background components. The filter calculates the optimum threshold separating those two classes so that their combined spread (intra-class variance) is minimal (see http://en.wikipedia.org/wiki/Otsu%27s_method).  Then the filter applies that threshold to the input image using the itkBinaryThresholdImageFilter. The numberOfHistogram bins can be set for the Otsu Calculator. The insideValue and outsideValue can be set for the BinaryThresholdImageFilter.  The filter produces a labeled volume.

The original reference is:

N.Otsu, A threshold selection method from gray level histograms, IEEE Trans.Syst.ManCybern.SMC-9,62–66 1979.

version: 0.1.0.$Revision: 19608 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/OtsuThresholdImageFilter

contributor: Bill Lorensen (GE)

acknowledgements: This command module was derived from Insight/Examples (copyright) Insight Software Consortium

"""

    input_spec = OtsuThresholdImageFilterInputSpec
    output_spec = OtsuThresholdImageFilterOutputSpec
    _cmd = "OtsuThresholdImageFilter "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}


class ResampleScalarVolumeInputSpec(CommandLineInputSpec):
    spacing = InputMultiPath(
        traits.Float,
        desc="Spacing along each dimension (0 means use input spacing)",
        sep=",",
        argstr="--spacing %s",
    )
    interpolation = traits.Enum(
        "linear",
        "nearestNeighbor",
        "bspline",
        "hamming",
        "cosine",
        "welch",
        "lanczos",
        "blackman",
        desc="Sampling algorithm (linear, nearest neighbor, bspline(cubic)  or windowed sinc). There are several sinc algorithms available as described in the following publication: Erik H. W. Meijering, Wiro J. Niessen, Josien P. W. Pluim, Max A. Viergever: Quantitative Comparison of Sinc-Approximating Kernels for Medical Image Interpolation. MICCAI 1999, pp. 210-217. Each window has a radius of 3;",
        argstr="--interpolation %s",
    )
    InputVolume = File(
        position=-2, desc="Input volume to be resampled", exists=True, argstr="%s"
    )
    OutputVolume = traits.Either(
        traits.Bool,
        File(),
        position=-1,
        hash_files=False,
        desc="Resampled Volume",
        argstr="%s",
    )


class ResampleScalarVolumeOutputSpec(TraitedSpec):
    OutputVolume = File(position=-1, desc="Resampled Volume", exists=True)


class ResampleScalarVolume(SEMLikeCommandLine):
    """title: Resample Scalar Volume

category: Legacy.Filtering

description: Resampling an image is an important task in image analysis. It is especially important in the frame of image registration. This module implements image resampling through the use of itk Transforms. This module uses an Identity Transform. The resampling is controlled by the Output Spacing. "Resampling" is performed in space coordinates, not pixel/grid coordinates. It is quite important to ensure that image spacing is properly set on the images involved. The interpolator is required since the mapping from one space to the other will often require evaluation of the intensity of the image at non-grid positions. Several interpolators are available: linear, nearest neighbor, bspline and five flavors of sinc. The sinc interpolators, although more precise, are much slower than the linear and nearest neighbor interpolator. To resample label volumnes, nearest neighbor interpolation should be used exclusively.

version: 0.1.0.$Revision: 20594 $(alpha)

documentation-url: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/ResampleVolume

contributor: Bill Lorensen (GE)

acknowledgements: This work is part of the National Alliance for Medical Image Computing (NAMIC), funded by the National Institutes of Health through the NIH Roadmap for Medical Research, Grant U54 EB005149.

"""

    input_spec = ResampleScalarVolumeInputSpec
    output_spec = ResampleScalarVolumeOutputSpec
    _cmd = "ResampleScalarVolume "
    _outputs_filenames = {"OutputVolume": "OutputVolume.nii"}



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
AFNI_ is a software suite for the analysis and display of anatomical and functional MRI data.

.. include:: ../../../doc/links_names.txt

"""

from .base import Info
from .preprocess import (
    AlignEpiAnatPy,
    Allineate,
    Automask,
    AutoTcorrelate,
    AutoTLRC,
    Bandpass,
    BlurInMask,
    BlurToFWHM,
    ClipLevel,
    DegreeCentrality,
    Despike,
    Detrend,
    ECM,
    Fim,
    Fourier,
    Hist,
    LFCD,
    Maskave,
    Means,
    OutlierCount,
    QualityIndex,
    ROIStats,
    Retroicor,
    Seg,
    SkullStrip,
    TCorr1D,
    TCorrMap,
    TCorrelate,
    TNorm,
    TProject,
    TShift,
    TSmooth,
    Volreg,
    Warp,
    QwarpPlusMinus,
    Qwarp,
)
from .svm import SVMTest, SVMTrain
from .utils import (
    ABoverlap,
    AFNItoNIFTI,
    Autobox,
    Axialize,
    BrickStat,
    Bucket,
    Calc,
    Cat,
    CatMatvec,
    CenterMass,
    ConvertDset,
    Copy,
    Dot,
    Edge3,
    Eval,
    FWHMx,
    LocalBistat,
    Localstat,
    MaskTool,
    Merge,
    Notes,
    NwarpApply,
    NwarpAdjust,
    NwarpCat,
    OneDToolPy,
    Refit,
    ReHo,
    Resample,
    TCat,
    TCatSubBrick,
    TStat,
    To3D,
    Unifize,
    Undump,
    ZCutUp,
    GCOR,
    Zcat,
    Zeropad,
)
from .model import Deconvolve, Remlfit, Synthesize
